import 'dart:convert';

import 'package:flutter/material.dart';
import 'package:spark_form/spark.dart';
import 'package:spark_form/spark_verification.dart';
import 'package:spark_form/spark_widgets.dart';
import 'package:spark_form/spark_modal.dart';

void main() {
  runApp(const MyApp());
}

class MyApp extends StatefulWidget {
  const MyApp({Key? key}) : super(key: key);

  @override
  State<MyApp> createState() => _MyAppState();
}

class _MyAppState extends State<MyApp> {
  FormGroupManage? _manageData;

  FormGroupManage get _manage => _manageData!;

  final Map<String, ManageFieldItem> _fieldMap = {
    'test': ManageFieldItem(data: '11'),
    'text2': ManageFieldItem(data: ''),
    'test1': ManageFieldItem(data: ''),
    'test2': ManageFieldItem(
        data: SparkLabelTitleData.from('hahshdh', 'change value')),
    'test3': ManageFieldItem(
        data: SparkLabelTitleData.from('hahshdh', 'change value')),
    'test4': ManageFieldItem(
        data: SparkLabelTitleData.from('hahshdh', 'change value')),
    'test5': ManageFieldItem(data: '2'),
    'test6': ManageFieldItem(data: '2'),
    'test7': ManageFieldItem(data: '2'),
    'select': ManageFieldItem(
      data: List.generate(
        5,
        (index) => SparkTagGroupItem.from(
          name: 'hh$index a',
          data: '$index',
        ),
      ),
    ),
    'select1': ManageFieldItem(
      data: List.generate(
        5,
        (index) => SparkTagGroupItem.from(
          name: 'hh$index a',
          data: '$index',
        ),
      ),
    ),
    'select2': ManageFieldItem(
      data: List.generate(
        5,
        (index) => SparkTagGroupItem.from(
          name: 'hh$index a',
          data: '$index',
        ),
      ),
    ),
  };

  final SparkFormController _controller = SparkFormController();

  void check() {
    _controller.validate();
  }

  void _handleSubmit(BuildContext context) {
    var result = _controller.validate(focusError: true);
    var data = _manage.getJson();
    print('get all data: $data');
    String resultData =
        'get all data:\n\n ${jsonEncode(data)}\n\n\n check result: ${result.success} field: ${result.fieldName} error: ${result.errorMessage}';
    showModalBottomSheet(
      context: context,
      builder: (_) => Center(
        child: Text(
          resultData,
          style: !result.success ? const TextStyle(color: Colors.red) : null,
        ),
      ),
    );
    // _manage.reset();
    // print('reset form data');
  }

  @override
  void initState() {
    _manageData = FormGroupManage.create(_fieldMap);
    super.initState();
  }

  void getDate(
      Function(String value) callback, BuildContext context, String format) {
    showSparkDatePicker<String>(
            context: context,
            format: format,
            behavior: DatePickerBehavior.confirmWithPop,
            defaultDate: DateTime(2020, 12, 16),
            maxDate: DateTime(2090, 12, 1, 13, 24, 12))
        .then((value) {
      if (value != null) {
        callback(value);
      }
    });
  }

  @override
  Widget build(BuildContext context) {
    double _labelWidth = 60.0;
    return MaterialApp(
      home: Scaffold(
        appBar: AppBar(
          backgroundColor: const Color(0xFF2878FF),
          title: const Text('Spark form demo'),
          elevation: 0,
        ),
        body: Builder(
            builder: (_ctx) => SafeArea(
                  child: Column(
                    children: [
                      Expanded(
                        child: SingleChildScrollView(
                          padding: const EdgeInsets.symmetric(horizontal: 14),
                          child: SparkForm(
                            controller: _controller,
                            manage: _manage,
                            child: Column(
                              children: [
                                SparkFormItem(
                                  rules: [
                                    SparkLengthValidator.length(
                                      10,
                                      errorMessage: 'length max',
                                      type: VerifyType.change,
                                      strategy: FormStrategy.limit,
                                    ),
                                    SparkNumberValidator.number(
                                      errorMessage: 'only support number',
                                      type: VerifyType.change,
                                      strategy: FormStrategy.tip,
                                    ),
                                    SparkNumberValidator.length(
                                      integerLength: 6,
                                      decimalLength: 2,
                                      errorMessage: 'length check',
                                      type: VerifyType.change,
                                      strategy: FormStrategy.tip,
                                    ),
                                    SparkNumberValidator.range(
                                      min: 6,
                                      max: 50,
                                      errorMessage: 'max check',
                                      type: VerifyType.blur,
                                      strategy: FormStrategy.tip,
                                    ),
                                    SparkRequiredValidator.required(
                                      errorMessage: 'is not empty',
                                      type: VerifyType.change,
                                      strategy: FormStrategy.tip,
                                    ),
                                  ],
                                  label: 'input',
                                  fieldName: 'test',
                                  child: SparkTextField(
                                    manage: _manage['test'],
                                    maxLines: 1,
                                    cleanble: true,
                                    // textDirection: TextDirection.rtl,
                                    hitText: 'please input something',
                                    contentPadding: const EdgeInsets.symmetric(
                                        horizontal: 10),
                                  ),
                                ),
                                SparkFormItem(
                                  readOnly: true,
                                  label: 'select',
                                  isRequired: true,
                                  padding: const EdgeInsets.symmetric(
                                      horizontal: 0, vertical: 10),
                                  rules: [
                                    SparkRequiredValidator.required(
                                      errorMessage: 'pick one',
                                      type: VerifyType.change,
                                      strategy: FormStrategy.tip,
                                    ),
                                  ],
                                  fieldName: 'test1',
                                  child: SparkSelect(
                                    manage: _manage['test1'],
                                    options: [
                                      SparkSelectOption.form(
                                          'first', 'first value'),
                                      SparkSelectOption.form(
                                          'last', 'last value')
                                    ],
                                  ),
                                  appendWidget: Container(
                                    child: const Text(
                                        'im append, show in first value'),
                                    alignment: Alignment.center,
                                  ),
                                  appendWidgetBuilder:
                                      (_ctx, value, Widget? cache) {
                                    if (value == 'first value') {
                                      return cache!;
                                    }
                                  },
                                ),
                                SparkFormItem(
                                  fieldName: 'test2',
                                  label: 'title',
                                  readOnly: true,
                                  child: SparkLabelTitle(
                                    manage: _manage['test2'],
                                    contentPadding:
                                        const EdgeInsets.only(right: 10),
                                    onTap: (_data, updater) {
                                      getDate((value) {
                                        _data.title = value;
                                        _data.data = value;
                                        updater(_data);
                                      },
                                          _ctx,
                                          SparkDateFormat
                                              .yearMonthDayAndHourMinuteSecond);
                                    },
                                  ),
                                ),
                                Row(
                                  mainAxisSize: MainAxisSize.max,
                                  mainAxisAlignment:
                                      MainAxisAlignment.spaceBetween,
                                  children: [
                                    Expanded(
                                      child: SparkFormItem(
                                        labelWidth: _labelWidth,
                                        fieldName: 'test3',
                                        label: 'titleRow',
                                        readOnly: true,
                                        child: SparkLabelTitle(
                                          manage: _manage['test3'],
                                          selectDown: true,
                                          contentPadding:
                                              const EdgeInsets.only(right: 10),
                                          padding:
                                              const EdgeInsets.only(right: 5),
                                          onTap: (_data, updater) {
                                            getDate((value) {
                                              _data.title = value;
                                              _data.data = value;
                                              updater(_data);
                                            }, _ctx,
                                                SparkDateFormat.yearMonthDay);
                                          },
                                        ),
                                      ),
                                      flex: 3,
                                    ),
                                    Container(
                                      width: 1,
                                      height: 14,
                                      color: const Color(0xFFcccccc),
                                    ),
                                    Expanded(
                                      child: SparkFormItem(
                                        labelWidth: 0,
                                        fieldName: 'test4',
                                        // label: 'titleRow',
                                        labelBuilder: (_, _t) => Container(),
                                        readOnly: true,
                                        child: SparkLabelTitle(
                                          manage: _manage['test4'],
                                          contentPadding:
                                              const EdgeInsets.only(right: 10),
                                          selectDown: true,
                                          onTap: (_data, updater) {
                                            _data.title = 'hanbsdhjb';
                                            _data.data = 'what\'s up';
                                            updater(_data);
                                          },
                                        ),
                                      ),
                                      flex: 1,
                                    ),
                                  ],
                                ),
                                SparkFormItem(
                                  fieldName: 'select',
                                  label: 'group',
                                  readOnly: true,
                                  crossCenter: false,
                                  isRequired: true,
                                  child: Row(
                                    crossAxisAlignment:
                                        CrossAxisAlignment.baseline,
                                    textBaseline: TextBaseline.ideographic,
                                    children: [
                                      Expanded(
                                        child: SparkTagGroup(
                                          manage: _manage['select'],
                                          type: SparkTagGroupType.multiple,
                                          hitText: 'you can add option',
                                          showClean: true,
                                        ),
                                        flex: 1,
                                      ),
                                      Container(
                                        width: 1,
                                        height: 14,
                                        color: const Color(0xFFcccccc),
                                        margin: const EdgeInsets.symmetric(
                                            horizontal: 10),
                                      ),
                                      GestureDetector(
                                        onTap: () {
                                          (_manage['select'].notifier.value
                                                  as List<SparkTagGroupItem>)
                                              .add(SparkTagGroupItem.from(
                                                  name: 'test'));
                                          _manage['select'].update();
                                        },
                                        child: const Icon(
                                          Icons.search,
                                          size: 18,
                                        ),
                                      ),
                                    ],
                                  ),
                                ),
                                SparkFormItem(
                                  fieldName: 'select1',
                                  label: 'mult',
                                  readOnly: true,
                                  crossCenter: false,
                                  isRequired: true,
                                  rules: [
                                    SparkRequiredValidator.required(
                                      errorMessage: 'pick one',
                                      type: VerifyType.all,
                                      strategy: FormStrategy.tip,
                                    ),
                                  ],
                                  child: SparkTagGroup(
                                    manage: _manage['select1'],
                                    type: SparkTagGroupType.multipleAndIncrease,
                                    hitText: 'add option',
                                    showClean: true,
                                  ),
                                ),
                                SparkFormItem(
                                  fieldName: 'select2',
                                  label: 'singe',
                                  readOnly: true,
                                  crossCenter: false,
                                  isRequired: true,
                                  child: SparkTagGroup(
                                    manage: _manage['select2'],
                                    type: SparkTagGroupType.singe,
                                    hitText: 'add option',
                                  ),
                                ),
                                SparkFormItem(
                                  fieldName: 'text2',
                                  label: 'text',
                                  isRequired: true,
                                  crossCenter: false,
                                  direction: Axis.vertical,
                                  rules: [
                                    SparkNumberValidator.length(
                                      integerLength: 10,
                                      decimalLength: 2,
                                      errorMessage: 'hhhh',
                                      type: VerifyType.all,
                                      strategy: FormStrategy.tip,
                                    )
                                  ],
                                  labelPadding:
                                      const EdgeInsets.only(bottom: 10),
                                  child: SparkTextField(
                                    manage: _manage['text2'],
                                    showConuter: true,
                                    minLines: 2,
                                    maxLines: 5,
                                    maxLength: 99,
                                  ),
                                ),
                              ],
                            ),
                          ),
                        ),
                        flex: 1,
                      ),
                      Builder(
                          builder: (_) => GestureDetector(
                                onTap: () {
                                  _handleSubmit(_);
                                },
                                child: Container(
                                  margin: const EdgeInsets.only(top: 30),
                                  width: MediaQuery.of(_).size.width - 40,
                                  height: 50,
                                  alignment: Alignment.center,
                                  decoration: const BoxDecoration(
                                    borderRadius:
                                        BorderRadius.all(Radius.circular(40)),
                                    color: Color(0xFF2878FF),
                                  ),
                                  child: const Text(
                                    'Submit',
                                    style: TextStyle(
                                        color: Colors.white, fontSize: 18.0),
                                  ),
                                ),
                              ))
                    ],
                  ),
                )),
      ),
    );
  }
}
