part of spark_modal;

Future<T?> showSparkModalSheet<T>({
  required BuildContext context,
  required WidgetBuilder builder,
  ShapeBorder? shape,
  bool useRootNavigator = false,
  Clip? clipBehavior,
}) {
  final NavigatorState navigator =
      Navigator.of(context, rootNavigator: useRootNavigator);
  return navigator.push(_SparkModalRoute<T>(
    builder: builder,
    shape: shape,
    clipBehavior: clipBehavior,
    barrierLabel: MaterialLocalizations.of(context).modalBarrierDismissLabel,
  ));
}

class _SparkModalRoute<T> extends PopupRoute<T> {
  _SparkModalRoute({
    this.barrierLabel,
    required this.builder,
    this.shape,
    this.clipBehavior,
    RouteSettings? settings,
  }) : super(settings: settings);

  WidgetBuilder builder;

  final ShapeBorder? shape;

  final Clip? clipBehavior;

  @override
  Duration get transitionDuration => const Duration(milliseconds: 180);

  @override
  bool get barrierDismissible => true;

  @override
  final String? barrierLabel;

  @override
  Color get barrierColor => Colors.black54;

  AnimationController? _animationController;

  @override
  AnimationController createAnimationController() {
    assert(_animationController == null);
    _animationController =
        BottomSheet.createAnimationController(navigator!.overlay!);
    return _animationController!;
  }

  @override
  Widget buildPage(BuildContext context, Animation<double> animation,
      Animation<double> secondaryAnimation) {
    Widget bottomSheet = MediaQuery.removePadding(
      context: context,
      removeTop: true,
      child: _SparkModalWrapper(
        animation: animation,
        builder: builder,
        shape: shape,
        clipBehavior: clipBehavior,
      ),
    );
    return InheritedTheme.captureAll(context, bottomSheet);
  }
}

class _SparkModalWrapper extends StatefulWidget {
  const _SparkModalWrapper({
    Key? key,
    required this.animation,
    required this.builder,
    this.shape,
    this.clipBehavior,
  }) : super(key: key);

  final Animation animation;

  final WidgetBuilder builder;

  final ShapeBorder? shape;

  final Clip? clipBehavior;

  @override
  _SparkModalWrapperState createState() => _SparkModalWrapperState();
}

class _SparkModalWrapperState extends State<_SparkModalWrapper> {
  @override
  Widget build(BuildContext context) {
    return AnimatedBuilder(
      animation: widget.animation,
      builder: (BuildContext context, Widget? child) {
        final double bottomPadding = MediaQuery.of(context).padding.bottom;
        return ClipRect(
          child: CustomSingleChildLayout(
            delegate: _BottomWidgetLayout(
              widget.animation.value,
              bottomPadding: bottomPadding,
            ),
            child: child,
          ),
        );
      },
      child: GestureDetector(
        child: Material(
          shape: widget.shape,
          clipBehavior: widget.clipBehavior ?? Clip.hardEdge,
          child: SafeArea(
            child: widget.builder(context),
          ),
        ),
      ),
    );
  }
}

class _BottomWidgetLayout extends SingleChildLayoutDelegate {
  _BottomWidgetLayout(
    this.progress, {
    this.itemCount,
    this.showTitleActions,
    this.bottomPadding = 0,
  });

  final double progress;
  final int? itemCount;
  final bool? showTitleActions;
  final double bottomPadding;

  @override
  BoxConstraints getConstraintsForChild(BoxConstraints constraints) {
    return BoxConstraints(
      minWidth: constraints.maxWidth,
      maxWidth: constraints.maxWidth,
      minHeight: 0.0,
      maxHeight: bottomPadding + constraints.maxHeight,
    );
  }

  @override
  Offset getPositionForChild(Size size, Size childSize) {
    final height = size.height - childSize.height * progress;
    return Offset(0.0, height);
  }

  @override
  bool shouldRelayout(_BottomWidgetLayout oldDelegate) {
    return progress != oldDelegate.progress;
  }
}
