part of spark_modal;

typedef SparkPickerBuilder = Widget Function(BuildContext context, int index);

typedef SparkPickerChanged = void Function(int index);

class SparkPicker extends StatelessWidget {
  const SparkPicker({
    Key? key,
    required this.controller,
    required this.itemHeight,
    required this.itemBuilder,
    required this.itemCount,
    this.onChanged,
    this.onChange,
    this.onStart,
    this.selectionOverlay,
  }) : super(key: key);

  /// picker controller
  final FixedExtentScrollController controller;

  /// single option height
  final double itemHeight;

  /// pick option builder
  final SparkPickerBuilder itemBuilder;

  /// item count
  final int itemCount;

  /// on changed, get last result
  final SparkPickerChanged? onChanged;

  /// scroll animation change index, get scrolling index value
  final SparkPickerChanged? onChange;

  final Function? onStart;

  /// selection overlay widget
  final Widget? selectionOverlay;

  void _selectedChangedWhenScrollEnd(int index) {
    if (onChanged != null) {
      onChanged!(index);
    }
  }

  void _selectedChangedWhenScrolling(int index) {
    if (onChange != null) {
      onChange!(index);
    }
  }

  @override
  Widget build(BuildContext context) {
    return NotificationListener(
      onNotification: (ScrollNotification notification) {
        if (notification.depth == 0 &&
            notification is ScrollEndNotification &&
            notification.metrics is FixedExtentMetrics) {
          final FixedExtentMetrics metrics =
              notification.metrics as FixedExtentMetrics;
          final int currentItemIndex = metrics.itemIndex;
          _selectedChangedWhenScrollEnd(currentItemIndex);
        }
        if (notification is ScrollStartNotification) {
          if (onStart != null) {
            onStart!();
          }
        }
        return false;
      },
      child: CupertinoPicker.builder(
        itemExtent: itemHeight,
        useMagnifier: true,
        selectionOverlay: selectionOverlay ??
            Container(
              decoration: const BoxDecoration(
                border: Border.symmetric(
                  horizontal: BorderSide(
                    color: Color(0xFFE5E5E5),
                  ),
                ),
              ),
            ),
        scrollController: controller,
        childCount: itemCount,
        itemBuilder: itemBuilder,
        onSelectedItemChanged: (int value) {
          _selectedChangedWhenScrolling(value);
        },
      ),
    );
  }
}

class SparkDatePickerData {
  SparkDatePickerData.form(
    this.label,
    this.value,
    this.index,
  );

  /// show label
  String label;

  /// value
  dynamic value;

  /// option index
  /// correspondence with the array
  int index;

  @override
  bool operator ==(Object other) {
    if (other is SparkDatePickerData) {
      return other.value == value && other.index == index;
    }
    return false;
  }

  @override
  int get hashCode => Object.hash(value, index, label);
}
