part of spark;

class FormGroupManage {
  FormGroupManage._(Map<String, ManageFieldItem> createField)
      : _saveRawData = createField {
    _setupManage(createField);
  }

  /// create manage
  factory FormGroupManage.create(Map<String, ManageFieldItem> createField) =>
      FormGroupManage._(createField);

  /// manage save
  final Map<String, FormGroupItem> _list = {};

  /// save init data
  final Map<String, ManageFieldItem> _saveRawData;

  /// get focus and controller info
  Map<String, FormGroupItem> get manageList => _list;

  FormGroupItem operator [](String index) {
    return _list[index]!;
  }

  /// setup field data
  void _setupManage(Map<String, ManageFieldItem> createField) {
    createField.forEach((key, value) {
      _list[key] = FormGroupItem.create(init: value.data ?? '');
    });
  }

  /// reset manage by default data
  void reset() {
    for (MapEntry<String, ManageFieldItem> _entry in _saveRawData.entries) {
      _list[_entry.key]!.notifier.updateValue(_entry.value.data);
    }
  }

  // /// clear manage data
  // void clear() {
  //   for (MapEntry<String, ManageFieldItem> _entry in _saveRawData.entries) {
  //     _list[_entry.key]!.notifier.updateValue(null);
  //   }
  // }

  /// get json data
  /// exclude field[excludeField]is not include in json data
  Map getJson({List<String>? excludeField}) {
    Map<String, dynamic> _jsonData = {};
    _list.forEach((key, value) {
      if (excludeField == null || !excludeField.contains(key)) {
        dynamic _fieldData = value.notifier.value;
        dynamic _value;
        if (_fieldData is SparkFormDataChangeImp) {
          _value = _fieldData.data;
        } else if (_fieldData is List<SparkFormDataChangeImp>) {
          _value = _fieldData
              .where((element) => element.needSelected)
              .map((e) => e.data)
              .toList();
        } else {
          _value = _fieldData;
        }
        _jsonData[key] = _value;
      }
    });
    return _jsonData;
  }

  /// dispose all
  void dispose() {
    for (var _recordItem in _list.values) {
      _recordItem.dispose();
    }
  }
}

class ManageFieldItem {
  ManageFieldItem({this.data});

  dynamic data;
}

class FormGroupItem {
  /// create manage
  FormGroupItem.create({dynamic init}) : oldValue = init {
    _notifier = SparkDataChangeNotifier(data: init);
    _focusNode = FocusNode();
  }

  SparkDataChangeNotifier? _notifier;
  FocusNode? _focusNode;
  VoidCallback? clickCallback;

  dynamic oldValue;

  SparkDataChangeNotifier get notifier => _notifier!;

  FocusNode get focusNode => _focusNode!;

  /// update value
  /// must use this function update value
  void updateValue(dynamic value, bool recordOldValue) {
    notifier.updateValue(value);
    if (recordOldValue) {
      oldValue = value;
    }
  }

  /// just update
  void update() {
    notifier.notifyAll();
  }

  /// register click callback
  /// will call on form click
  /// see [FocusPointerScope]
  void registerClickCallback(VoidCallback callback) {
    clickCallback = callback;
  }

  /// clean callback
  void clearClickCallback() {
    clickCallback = null;
  }

  /// dispose all
  void dispose() {
    if (_notifier != null) {
      SparkDataChangeNotifier _temp = _notifier!;
      _notifier = null;
      _temp.dispose();
    }
    if (_focusNode != null) {
      FocusNode _temp = _focusNode!;
      _focusNode = null;
      _temp.dispose();
    }
    clickCallback = null;
  }
}

class SparkDataChangeNotifier extends ChangeNotifier
    implements ValueListenable {
  SparkDataChangeNotifier({dynamic data})
      : _data = data,
        super();

  dynamic _data;

  @override
  get value => _data;

  void notifyAll() {
    notifyListeners();
  }

  void updateValue(dynamic data) {
    _data = data;
    notifyListeners();
  }
}

abstract class SparkFormDataChangeImp {
  SparkFormDataChangeImp(this.data);

  /// Serialized data
  /// [FormManage.getJson] use this filed to create json data
  dynamic data;

  /// only true can use by [FormManage.getJson]
  bool needSelected = false;
}

class FormDataChangeNotification extends Notification {
  /// form data change notification
  FormDataChangeNotification({
    this.changeField,
    required this.newValue,
  });

  /// form field
  String? changeField;

  /// form item widget new value
  dynamic newValue;
}

class SparkFormData extends InheritedWidget {
  const SparkFormData({
    required this.formData,
    Key? key,
    required Widget child,
  }) : super(key: key, child: child);

  final SparkFormItemData formData;

  /// get [SparkFormItemData] of context
  static SparkFormItemData? of(BuildContext context) {
    return context
        .dependOnInheritedWidgetOfExactType<SparkFormData>()
        ?.formData;
  }

  @override
  bool updateShouldNotify(covariant InheritedWidget oldWidget) {
    return true;
  }
}

class SparkFormItemData {
  Map<String, ValidateCallback> validateList = <String, ValidateCallback>{};

  FormGroupManage? _manage;

  FormGroupManage get manage => _manage!;

  void _setManage(FormGroupManage manage) {
    _manage = manage;
  }

  /// when bind widget dispose call it's
  void disposeField(String field) {
    validateList.remove(field);
  }

  /// clear all bind
  void dispose() {
    validateList.clear();
  }
}

class SparkFormController {
  SparkFormItemData? _data;

  FormGroupManage? _manage;

  FormGroupManage get manage => _manage!;

  SparkFormItemData get data => _data!;

  /// use in form init, set controller data
  void _setFormItemData(SparkFormItemData data, FormGroupManage manageData) {
    _data = data;
    _manage = manageData;
  }

  ValidateResult validateField(String fieldName, VerifyType type,
      {dynamic preValue}) {
    ValidateResult Function(dynamic, VerifyType)? _fieldCheck =
        data.validateList[fieldName];
    FormGroupItem _fieldData = manage[fieldName];
    assert(_fieldCheck != null,
        'get null check rule, please check form item fieldName');
    return _fieldCheck!(preValue ?? _fieldData.notifier.value, type);
  }

  /// validate all rules, get validate result
  /// [focusError] auto focus first error
  ValidateResult validate({bool? focusError}) {
    Map<String, ValidateResult Function(dynamic, VerifyType)> _list =
        data.validateList;
    for (MapEntry<String, ValidateResult Function(dynamic, VerifyType)> _item
        in _list.entries) {
      FormGroupItem _fieldData = manage[_item.key];
      ValidateResult _result = _item.value(
        _fieldData.notifier.value,
        VerifyType.all,
      );
      if (!_result.success) {
        if (focusError == true) {
          _fieldData.focusNode.requestFocus();
        }
        return _result;
      }
    }
    return ValidateResult.from(success: true);
  }

  /// reset all field data
  void resetAll() {
    _manage?.reset();
  }

  /// clean all data
  void cleanAll() {
    // _manage?.clear();
  }
}

abstract class SparkOption {
  SparkOption(this._label, this._value);

  final String? _label;

  final String? _value;

  String get label => _label!;

  String get value => _value!;
}
