part of spark;

const Color _defaultFontColor = Color(0xFF191C27);

/// default primary color
const Color _primaryDefaultColor = Color(0xFF2878FF);

/// label width
const double _defaultLabelWidth = 60;

/// border color
const Color _defaultBorderColor = Color(0xFF2878FF);

/// hit text color
const Color _hitTextColor = Color(0xFF9AA0B2);

/// error color
const Color _errorOrRequiredColor = Color(0xFFe74c3c);

/// default font size
const double _defaultFormItemFontSize = 15.0;

/// default form color
const Color _defaultFormItemColor = Color(0xFF333333);

/// all form item disabled color
const Color _defaultFormItemDisabledColor = Color(0xFF999999);

/// select option min size
const Size _defaultSelectItemSize = Size(76, 30);

/// spark select option selected style
const SparkSelectDecoration _defaultSelectedDecoration = SparkSelectDecoration(
  fontSize: _baseFontSize,
  textColor: _primaryDefaultColor,
  backgroundColor: Color(0x1A1677FF),
  border: Border.fromBorderSide(BorderSide(color: _primaryDefaultColor, width: 1)),
  borderRadius: BorderRadius.all(Radius.circular(20)),
);

/// spark select option unSelected style
const SparkSelectDecoration _defaultUnSelectedDecoration =
    SparkSelectDecoration(
  fontSize: _baseFontSize,
  textColor: Color(0xff646474),
  backgroundColor: Color(0xffF6F8FA),
  border: Border.fromBorderSide(BorderSide(color: Color(0xffF6F8FA), width: 1)),
  borderRadius: BorderRadius.all(Radius.circular(20)),
);

/// form item content box decoration
const FocusDecoration _defaultFormDecoration = FocusDecoration(
  focusBorder: Border(bottom: BorderSide(color: _defaultBorderColor, width: 1)),
  unFocusBorder: Border(bottom: BorderSide(color: Color(0xFFE8EBF3), width: 1)),
);

/// form item content padding
const EdgeInsets _defaultFormItemPadding =
    EdgeInsets.symmetric(horizontal: 0, vertical: 14);

/// form item content margin
const EdgeInsets _defaultFormItemMargin = EdgeInsets.zero;

/// default font size
const double _baseFontSize = 13.0;

const BoxDecoration _defaultTagDecoration = BoxDecoration(
    color: Color(0xFFF5F5F5),
    borderRadius: BorderRadius.all(Radius.circular(4)));

const BoxDecoration _defaultTagSelectedDecoration = BoxDecoration(
    color: _primaryDefaultColor,
    borderRadius: BorderRadius.all(Radius.circular(4)));

const TextStyle _defaultTagItemStyle =
    TextStyle(fontSize: _baseFontSize, color: Color(0xFF5F6472));

const TextStyle _defaultSelectedItemStyle =
    TextStyle(fontSize: _baseFontSize, color: Color(0xFFFFFFFF));

const double _defaultTagWidth = 80.0;

const EdgeInsets? _defaultTagContentPadding = EdgeInsets.symmetric(horizontal: 14, vertical: 7);

const Decoration _defaultTagAddDecoration = DashedDecoration(
  borderRadius: BorderRadius.all(Radius.circular(6)),
  color: Color(0xFFFFFFFF),
  dashedColor: Color(0xFFCCCCCC),
  gap: 2.5,
);

class SparkThemeData extends InheritedWidget {
  /// form theme data manage provider
  const SparkThemeData({required Widget child, required this.theme, Key? key})
      : super(child: child, key: key);

  /// form theme
  final SparkTheme theme;

  /// get form theme data of context
  static SparkTheme? of(BuildContext context) {
    return context.dependOnInheritedWidgetOfExactType<SparkThemeData>()?.theme;
  }

  @override
  bool updateShouldNotify(covariant InheritedWidget oldWidget) {
    return true;
  }
}

class SparkTheme {
  /// spark form theme
  /// default use in [SparkForm]
  const SparkTheme({
    this.labelColor = _defaultFontColor,
    this.hitColor = _hitTextColor,
    this.labelFontSize = _defaultFormItemFontSize,
    this.labelFontWeight = FontWeight.w500,
    this.requiredColor = _errorOrRequiredColor,
    this.requiredPrefix = '*',
    this.formItemFontSize = _defaultFormItemFontSize,
    this.formItemColor = _defaultFormItemColor,
    this.formItemDisabledColor = _defaultFormItemDisabledColor,
    this.labelWidth = _defaultLabelWidth,
    this.labelPadding = EdgeInsets.zero,
    this.formItemDecoration = _defaultFormDecoration,
    this.formItemPadding = _defaultFormItemPadding,
    this.formItemMargin = _defaultFormItemMargin,
    this.selectItemSize = _defaultSelectItemSize,
    this.selectedItemDecoration = _defaultSelectedDecoration,
    this.unSelectedItemDecoration = _defaultUnSelectedDecoration,
    this.errorColor = _errorOrRequiredColor,
    this.tagItemDecoration = _defaultTagDecoration,
    this.tagSelectedItemDecoration = _defaultTagSelectedDecoration,
    this.tagStyle = _defaultTagItemStyle,
    this.tagSelectedStyle = _defaultSelectedItemStyle,
    this.tagMinWidth = _defaultTagWidth,
    this.tagAddDecoration = _defaultTagAddDecoration,
    this.tagContentPadding = _defaultTagContentPadding,
    this.hitFontSize = 12,
  });

  final double? hitFontSize;

  final double? labelWidth;

  final String? requiredPrefix;

  final Color? requiredColor;

  final Color? labelColor;

  final double? labelFontSize;

  final FontWeight? labelFontWeight;

  final Color? hitColor;

  final double? formItemFontSize;

  final Color? formItemColor;

  final Color? formItemDisabledColor;

  final EdgeInsets? labelPadding;

  final FocusDecoration? formItemDecoration;

  final EdgeInsets? formItemPadding;

  final EdgeInsets? formItemMargin;

  final Size? selectItemSize;

  final SparkSelectDecoration? selectedItemDecoration;

  final SparkSelectDecoration? unSelectedItemDecoration;

  final Color? errorColor;

  final BoxDecoration? tagItemDecoration;

  final BoxDecoration? tagSelectedItemDecoration;

  final TextStyle? tagStyle;

  final TextStyle? tagSelectedStyle;

  final double? tagMinWidth;

  final Decoration? tagAddDecoration;

  final EdgeInsets? tagContentPadding;
}
