part of spark_widget;

class SparkSelect extends SparkFormStateLessWidget<String> {
  const SparkSelect({
    Key? key,
    required FormGroupItem manage,
    required this.options,
    this.itemBuilder,
    this.itemSpacing,
    this.itemSize,
    this.selectedDecoration,
    this.unSelectedDecoration,
    this.alignment,
  }) : super(key: key, manage: manage);

  /// option data
  final List<SparkSelectOption> options;

  /// custom item builder
  final ItemValueBuilder? itemBuilder;

  /// item space
  final double? itemSpacing;

  /// set up option size
  /// use BoxConstraints
  final Size? itemSize;

  /// set up select option style
  final SparkSelectDecoration? selectedDecoration;

  /// set up default option style
  final SparkSelectDecoration? unSelectedDecoration;

  /// option alignment
  final WrapAlignment? alignment;

  Widget _buildOption(
    BuildContext context,
    SparkSelectOption option,
    String value,
    ValueUpdater updater,
    SparkTheme? theme,
  ) {
    bool _isChecked = option.value == value;
    SparkSelectDecoration _decoration = _isChecked
        ? (selectedDecoration ?? theme!.selectedItemDecoration!)
        : (unSelectedDecoration ?? theme!.unSelectedItemDecoration!);
    return GestureDetector(
      onTap: () {
        // updater(option.value);
        handleFormDataChange(context, option.value);
      },
      child: itemBuilder != null
          ? itemBuilder!(context, option, _decoration)
          : _buildDefaultOption(
              context, option.label, _isChecked, theme, _decoration),
    );
  }

  Widget _buildDefaultOption(BuildContext context, String optionName,
      bool isChecked, SparkTheme? theme, SparkSelectDecoration decoration) {
    Size _optionSize = itemSize ?? theme?.selectItemSize ?? const Size(76, 30);
    return Container(
      constraints: BoxConstraints(
          maxWidth: _optionSize.width, minHeight: _optionSize.height),
      alignment: Alignment.center,
      decoration: BoxDecoration(
        color: decoration.backgroundColor,
        borderRadius: decoration.borderRadius,
        border: decoration.border,
      ),
      child: Text(
        optionName,
        style: TextStyle(
          color: decoration.textColor,
          fontSize: decoration.fontSize,
          fontWeight: decoration.fontWeight,
        ),
      ),
    );
  }

  @override
  Widget build(BuildContext context) {
    SparkTheme? _theme = SparkThemeData.of(context);
    return SparkWrapper(
      manage: manage,
      builder: (_, _value, _updater) {
        return Wrap(
          spacing: itemSpacing ?? 10,
          alignment: alignment ?? WrapAlignment.end,
          children: [
            for (SparkSelectOption _item in options)
              _buildOption(context, _item, _value, _updater, _theme)
          ],
        );
      },
    );
  }
}

class SparkSelectOption extends SparkOption {
  SparkSelectOption.form(String label, String value, {this.data})
      : super(label, value);

  final dynamic data;
}

class SparkSelectDecoration {
  const SparkSelectDecoration({
    this.fontWeight,
    this.textColor,
    this.borderRadius,
    this.fontSize,
    this.backgroundColor,
    this.border,
  });

  SparkSelectDecoration copyWith({
    FontWeight? fontWeight,
    Color? textColor,
    BorderRadiusGeometry? borderRadius,
    double? fontSize,
    Color? backgroundColor,
    BoxBorder? border,
  }) {
    return SparkSelectDecoration(
      fontWeight: fontWeight ?? this.fontWeight,
      textColor: textColor ?? this.textColor,
      borderRadius: borderRadius ?? this.borderRadius,
      fontSize: fontSize ?? this.fontSize,
      backgroundColor: backgroundColor ?? this.backgroundColor,
      border: border ?? this.border,
    );
  }

  final FontWeight? fontWeight;
  final Color? textColor;
  final BorderRadiusGeometry? borderRadius;
  final double? fontSize;
  final Color? backgroundColor;
  final BoxBorder? border;
}
